@@ -29,6 +29,13 @@ centrality <- function(graph,alpha=1,posfun=abs)
 #   }
 
   W <- getWmat(graph)
+  
+  # If is list, compute for all:
+  if (is.list(W))
+  {
+    return(lapply(W,centrality, alpha=alpha,posfun=posfun))
+  }
+  
   n <- nrow(W)
   
   # Remove diagonal:
